package com.gesoft.czreal.common.base;

import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;

import java.util.List;

/**
 * @author WJZ
 * @name BaseFragment
 * @date 2018/6/24
 */

public abstract class BaseFragment<F extends BaseFragment> extends Fragment {
    private Handler         mHandler;
    private Handler         tHandler;
    private List<AsyncTask> list;
    private F fragment;
    private String tag;
    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        fragment = setFragment();
        list = createList();
        tag = this.getClass().getSimpleName();
    }

    public String tag() {
        return tag;
    }

    protected Handler getAutoLifeCyclemHandler() {
        return mHandler;
    }

    protected Handler getAutoLifeCycletHandler() {
        return tHandler;
    }

    protected void setAutoLifeCyclemHandler(Handler handler) {
        mHandler = handler;
    }

    protected void setAutoLifeCycletHandler(Handler handler) {
        tHandler = handler;
    }

    public List<AsyncTask> getAsyncTaskList() {
        return list;
    }

    protected abstract List<AsyncTask> createList();

    protected abstract F setFragment();

    public F getFragment() {
        return fragment;
    }

    protected void addAutoLifecycleAsyncTask(AutoLifecycleAsyncTaskBuilder asyncTask) {
        list.add(asyncTask.build());
    }

    public interface AutoLifecycleAsyncTaskBuilder {
        AsyncTask build();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();

        if (mHandler != null) {
            mHandler.removeCallbacksAndMessages(null);
        }
        if (tHandler != null) {
            tHandler.removeCallbacksAndMessages(null);
        }

        if (list != null && list.size() > 0) {
            for (AsyncTask asyncTask : list) {
                if (!asyncTask.isCancelled() && asyncTask.getStatus() != AsyncTask.Status.FINISHED) {
                    asyncTask.cancel(true);
                    list.remove(asyncTask);
                }
            }
            list.clear();
            list = null;
        }
    }
}
